/**
 * 处理单个reducer切片工具函数，消除switch case样板代码
 * @param  {any} initialState    初始状态
 * @param  {Object} handlers     处理对象，属性key为每个case type，value为处理相应function
 * @return {Object}              返回处理完毕的state
 * @example
 * import caseReducer from './caseReducer'
 *
 * const initialState = {
 *   data,
 * }
 * function getData(state, action) {
 *   return {
 *     ...state,
 *     data: action.response.data,
 *   }
 * }
 *
 * function postData(state, action) {
 *   return {
 *     ...state,
 *     data: action.response.data,
 *   }
 * }
 *
 * export default caseReducer(initialState, {
 *   GETDATA: getData,
 *   POSTDATA: postData,
 * })
 */
export default (initialState, handlers) => (state = initialState, action) => {
  if (Object.prototype.hasOwnProperty.call(handlers, action.type)) {
    return handlers[action.type](state, action)
  }
  return state
}
